namespace Commonmark.Internal {

    public class BlockQuoteParser : Parser.Block.AbstractBlockParser {

        private Node.BlockQuote block;

        construct {
            block = new Node.BlockQuote();
        }

        public override bool is_container() {
            return true;
        }

        public override bool can_contain(Node.Block block) {
            return true;
        }

        public override Node.Block get_block() {
            return block;
        }

        public override Parser.Block.BlockContinue
            try_continue(Parser.Block.ParserState state) {
            int next_non_space = state.get_next_non_space_index();
            if (is_marker(state, next_non_space)) {
                int new_column = state.get_column() + state.get_indent() + 1;
                /* optional following space or tab */
                if (Internal.Util.Parsing.is_space_or_tab(
                        state.get_line().content, next_non_space + 1)) {
                    new_column++;
                }
                return Parser.Block.BlockContinue.at_column(new_column);
            } else {
                return Parser.Block.BlockContinue.none();
            }
        }

        private static bool
            is_marker(Parser.Block.ParserState state, int index) {
            var line = state.get_line().content;
            return state.get_indent() < Internal.Util.Parsing.CODE_BLOCK_INDENT &&
                index < line.length && line.get(index) == '>';
        }

        public class Factory : Parser.Block.AbstractBlockParserFactory {

            public override Parser.Block.BlockStart
                try_start(Parser.Block.ParserState state,
                          Parser.Block.MatchedBlockParser matched_block_parser) {
                int next_non_space = state.get_next_non_space_index();
                if (is_marker(state, next_non_space)) {
                    int new_column = state.get_column() +
                        state.get_indent() + 1;
                    /* optional following space or tab */
                    if (Internal.Util.Parsing.is_space_or_tab(
                            state.get_line().content,
                            next_non_space + 1)) {
                        new_column++;
                    }
                    return Parser.Block.BlockStart.from(
                        new Internal.BlockQuoteParser())
                        .at_column(new_column);
                } else {
                    return Parser.Block.BlockStart.none();
                }
            }
        }
    }
}
