namespace Commonmark.Node {

    /**
     * A link with a destination and an optional title; the link text is in
     * child nodes.
     *
     * Example for an inline link in a CommonMark document:
     *
     * <pre><code>
     * [link](/uri "title")
     * </code></pre>
     *
     * The corresponding Link node would look like this:
     *
     * <ul>
     * <li>{@link #getDestination()} returns {@code "/uri"}
     * <li>{@link #getTitle()} returns {@code "title"}
     * <li>A {@link Text} child node with {@link Text#getLiteral()
     *     getLiteral} that returns {@code "link"}</li>
     * </ul>
     *
     * Note that the text in the link can contain inline formatting, so it could
     * also contain an Image} or Emphasis, etc.
     *
     * http://spec.commonmark.org/0.26/#links
     */
    public class Link : Node {

        public string destination { get; set; }
        public string title { get; set; }

        public Link() {}

        public Link.full(string destination, string? title) {
            this.destination = destination;
            this.title = title;
        }

        public override void accept(Visitor visitor) {
            visitor.visit(this);
        }

        protected override string to_string_attributes() {
            return "destination=" + destination + ", title=" + title;
        }
    }
}
