namespace Commonmark.Parser {

    /**
     * A line or part of a line from the input source.
     */
    public class SourceLine : GLib.Object {

        public string content { get; private set; }
        public Node.SourceSpan source_span { get; private set; }

        public static SourceLine from(string content,
                                      Node.SourceSpan? source_span) {
            return new SourceLine(content, source_span);
        }

        private SourceLine(string content, Node.SourceSpan? source_span) {
            this.content = content;
            this.source_span = source_span;
        }

        public SourceLine substring(int begin_index, int end_index) {
            string new_content = content[begin_index:end_index];
            Node.SourceSpan new_source_span = null;
            if (source_span != null) {
                int column_index = source_span.column_index + begin_index;
                int length = end_index - begin_index;
                if (length != 0) {
                    new_source_span = Node.SourceSpan.from(source_span.line_index,
                                                           column_index, length);
                }
            }
            return SourceLine.from(new_content, new_source_span);
        }
    }
}
