namespace Commonmark.Renderer.Html {

    /**
     * Sanitizes urls for img and a elements by whitelisting protocols.
     *
     * This is intended to prevent XSS payloads like [Click this totally safe
     * url](javascript:document.xss=true;)
     *
     * Implementation based on https://github.com/OWASP/java-html-sanitizer/blob/f07e44b034a45d94d6fd010279073c38b6933072/src/main/java/org/owasp/html/FilterUrlByProtocolAttributePolicy.java
     */
    public interface UrlSanitizer : GLib.Object {

        /**
         * Sanitize a url for use in the href attribute of a Link.
         *
         * @param url Link to sanitize
         * @return Sanitized link
         */
        public abstract string sanitize_link_url(string url);

        /**
         * Sanitize a url for use in the src attribute of a Image.
         *
         * @param url Link to sanitize
         * @return Sanitized link Image
         */
        public abstract string sanitize_image_url(string url);
    }
}
